local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true)
local me = {}

NTWatch = {}

function NTWatch:New(parent)
    local ret = CloneTable(me)
    ret.parent = parent
    return ret
end

function me:GetRollStatistics()
    local db = self.parent.db
    return GetSubtable(db.char, "RollStatistics")
end

function me:GetRollHistory()
    local db = self.parent.db
    return GetSubtable(db.char, "RollHistory")
end

function me:AddRollStatistics(player, choice)
    local db = self.parent.db
    local stat = GetSubtable(db.char, "RollStatistics")

    local n, data, found
    for n, data in pairs(stat) do
        if data.player == player then
            found = true
            data[choice] = data[choice] + 1
            break
        end
    end
    if not found then
        table.insert(stat, { player = player, win = 0, need = 0, greed = 0, disenchant = 0, pass = 0, [choice] = 1 })
    end

    if self.parent.gui then
        self.parent.gui:UpdateRollStatistics()
    end
end

function me:AddRollHistory(item, res, choice, player, roll)
    local db = self.parent.db
    local hist = GetSubtable(db.char, "RollHistory")

    table.insert(hist, 1, { item = item, res = res, choice = choice, player = player, roll = roll })

    if self.parent.gui then
        self.parent.gui:UpdateRollHistory()
    end
end

function me:ClearRollStatistics()
    local db = self.parent.db

    db.char.RollStatistics = {}
    db.char.RollHistory = {}

    if self.parent.gui then
        self.parent.gui:UpdateRollStatistics()
        self.parent.gui:UpdateRollHistory()
    end
end

function me:EventAllPassed(item)
    local db = self.parent.db
    local res = GetSubtable(db.char, "RollResults", item)

    db.char.RollResults[item] = nil

    self:AddRollHistory(item, res, "pass")

    if self.parent.reporter then
        self.parent.reporter:EventAllPassed(item)
    end
end

function me:EventChose(item, player, choice)
    local db = self.parent.db
    GetSubtable(db.char, "RollResults", item, player).choice = choice

    self:AddRollStatistics(player, choice)

    if choice == "need" and player ~= UnitName("player") then
        if self.parent.reporter then
            self.parent.reporter:EventChoseNeed(item, player)
        end
    end
end

function me:EventRolled(item, player, choice, roll)
    local db = self.parent.db

    local res = GetSubtable(db.char, "RollResults", item, player)
    res.choice, res.roll = choice, roll
end

function me:EventWon(item, player)
    local db = self.parent.db
    local res = GetSubtable(db.char, "RollResults", item, player)

    self:EventWonNoSpam(item, player, res.choice or "unknown", res.roll or "?")
end

function me:EventWonNoSpam(item, player, choice, roll)
    local db = self.parent.db
    local res = GetSubtable(db.char, "RollResults", item)

    db.char.RollResults[item] = nil

    self:AddRollHistory(item, res, choice, player, roll)
    self:AddRollStatistics(player, "win")

    if self.parent.reporter then
        self.parent.reporter:EventWon(item, player, choice, roll)
    end
end

function me:EventJoinGroup()
    self.parent.db.char.RollResults = {}
    if self.parent.gui then
        self.parent.gui:ConfirmClearStatistics()
    else
        self:ClearRollStatistics()
    end
end

function me:EventLeaveGroup()
    self.parent.db.char.RollResults = {}
end

function me:EventEnterInstance()
    if self.parent.gui then
        self.parent.gui:ConfirmClearStatistics()
    else
        self:ClearRollStatistics()
    end
end
